from .account import AccountResource
from .base import BaseResource
from .boards import BoardResource
from .complexity import ComplexityResource
from .groups import GroupResource
from .items import ItemResource
from .notification import NotificationResource
from .tags import TagResource
from .updates import UpdateResource
from .users import UserResource
from .workspaces import WorkspaceResource
from .webhooks import WebhooksResource

__all__ = [
    'ItemResource',
    'UpdateResource',
    'TagResource',
    'BoardResource',
    'UserResource',
    'GroupResource',
    'ComplexityResource',
    'WorkspaceResource',
    'NotificationResource',
    'AccountResource',
    'BaseResource',
    'WebhooksResource'
]



import time

import requests
import json


class GraphQLClient:
    def __init__(self, endpoint):
        self.endpoint = endpoint
        self.token = None
        self.header_name = None

    def execute(self, query, variables=None):
        query_type = 'query' if 'query' in query else 'mutation'
        stripped_query = query.strip().lstrip(f"{query_type} {{")
        query_with_complexity = f"""{query_type} {{
          complexity {{
            reset_in_x_seconds,
            after,
            before,
            query
          }}
          {stripped_query}
        """
        r = self._send(query_with_complexity, variables)
        if 'errors' in r:
            raise Exception(', '.join([e['message'] if 'message' in e else str(e) for e in r['errors']]))

        # rate limiting
        complexity = r['data']['complexity']
        while complexity['before'] - complexity['query'] < 0:
            time.sleep(complexity['reset_in_x_seconds'] + 1)
            r = self._send(query_with_complexity, variables)
            complexity = r['data']['complexity']
        if r.status_code >= 400:
            raise Exception(r.text)
        if 'errors' in r:
            raise Exception(', '.join([e['message'] if 'message' in e else str(e) for e in r['errors']]))

    def inject_token(self, token, header_name='Authorization'):
        self.token = token
        self.header_name = header_name

    def _send(self, query, variables):
        payload = {'query': query}
        headers = {}
        files = None

        if self.token is not None:
            headers[self.header_name] = '{}'.format(self.token)

        if variables is None:
            headers['Content-Type'] = 'application/json'
            payload = json.dumps({'query': query}).encode('utf-8')
        elif variables.get('file', None) is not None:
            headers['content'] = 'multipart/form-data'
            files = [
                ('variables[file]', (variables['file'], open(variables['file'], 'rb')))
            ]

        try:
            response = requests.request("POST", self.endpoint, headers=headers, data=payload, files=files)
            response.raise_for_status()
            return response.json()
        except requests.HTTPError as e:
            print(e)
            raise e



from monday.resources.base import BaseResource
from monday.query_joins import create_notification_query


class NotificationResource(BaseResource):
    def __init__(self, token):
        super().__init__(token)

    def create_notification(self, user_id, target_id, text, target_type):
        """
        Refer to here for more information -> https://api.developer.monday.com/docs/notification-queries
        """
        query = create_notification_query(user_id, target_id, text, target_type)
        return self.client.execute(query)



from monday.resources.base import BaseResource
from monday.query_joins import mutate_item_query, get_item_query, update_item_query, get_item_by_id_query, \
    update_multiple_column_values_query, mutate_subitem_query, add_file_to_column_query, delete_item_query


class ItemResource(BaseResource):
    def __init__(self, token):
        super().__init__(token)

    def create_item(self, board_id, group_id, item_name, column_values=None,
                    create_labels_if_missing=False):
        query = mutate_item_query(board_id, group_id, item_name, column_values,
                                  create_labels_if_missing)
        return self.client.execute(query)

    def create_subitem(self, parent_item_id, subitem_name, column_values=None,
                       create_labels_if_missing=False):
        query = mutate_subitem_query(parent_item_id, subitem_name, column_values,
                                     create_labels_if_missing)
        return self.client.execute(query)

    def fetch_items_by_column_value(self, board_id, column_id, value):
        query = get_item_query(board_id, column_id, value)
        return self.client.execute(query)

    def fetch_items_by_id(self, ids):
        query = get_item_by_id_query(ids)
        return self.client.execute(query)

    def change_item_value(self, board_id, item_id, column_id, value):
        query = update_item_query(board_id, item_id, column_id, value)
        return self.client.execute(query)

    def change_multiple_column_values(self, board_id, item_id, column_values, create_labels_if_missing=False):
        query = update_multiple_column_values_query(
            board_id, item_id, column_values, create_labels_if_missing)
        return self.client.execute(query)

    def add_file_to_column(self, item_id, column_id, file):
        query = add_file_to_column_query(item_id, column_id)
        return self.file_upload_client.execute(query, variables={'file': file})

    def delete_item_by_id(self, item_id):
        query = delete_item_query(item_id)
        return self.client.execute(query)



from .base import BaseResource
from ..query_joins import get_complexity_query


class ComplexityResource(BaseResource):
    def __init__(self, token):
        super().__init__(token)

    def get_complexity(self):
        query = get_complexity_query()
        return self.client.execute(query)



from monday.resources.base import BaseResource
from monday.query_joins import get_apps_monetization_supported_query, get_app_subscription_query, get_account_query, get_self_query


class AccountResource(BaseResource):
    def __init__(self, token):
        super().__init__(token)

    def get_apps_monetization_supported(self):
        query = get_apps_monetization_supported_query()
        return self.client.execute(query)

    def get_app_subscription(self):
        query = get_app_subscription_query()
        return self.client.execute(query)

    def get_account(self):
        query = get_account_query()
        return self.client.execute(query)

    def get_self(self):
        query = get_self_query()
        return self.client.execute(query)



from monday.query_joins import (
    create_board_by_workspace_query,
    get_board_items_by_id_query, get_board_items_query,
    get_boards_by_id_query,
    get_boards_query,
    get_columns_by_board_query,
    get_board_activity_query, get_paginated_board_items_query,
)
from monday.resources.base import BaseResource


class BoardResource(BaseResource):
    def __init__(self, token):
        super().__init__(token)

    def fetch_boards(self, **kwargs):
        query = get_boards_query(**kwargs)
        return self.client.execute(query)

    def fetch_boards_by_id(self, board_ids):
        query = get_boards_by_id_query(board_ids)
        return self.client.execute(query)

    def fetch_items_by_board_id(self, board_ids):
        query = get_board_items_query(board_ids)
        return self.client.execute(query)

    def fetch_paginated_items_by_board_id(self, board_ids, page, limit):
        query = get_paginated_board_items_query(board_ids, page, limit)
        return self.client.execute(query)

    def fetch_items_by_board_id_and_item_id(self, board_ids, item_id):
        query = get_board_items_by_id_query(board_ids, item_id)
        return self.client.execute(query)

    def fetch_columns_by_board_id(self, board_ids):
        query = get_columns_by_board_query(board_ids)
        return self.client.execute(query)

    def create_board(self, board_name, board_kind, workspace_id):
        query = create_board_by_workspace_query(board_name, board_kind, workspace_id)
        return self.client.execute(query)

    def get_board_activity(self, board_id, from_date, to_date, page, limit):
        query = get_board_activity_query(board_id, from_date, to_date, page, limit)
        return self.client.execute(query)



from monday.resources.base import BaseResource
from monday.query_joins import get_groups_by_board_query, get_items_by_group_query, create_group_query, \
    duplicate_group_query, archive_group_query, delete_group_query


class GroupResource(BaseResource):
    def __init__(self, token):
        super().__init__(token)

    def get_groups_by_board(self, board_ids):
        query = get_groups_by_board_query(board_ids=board_ids)
        return self.client.execute(query)

    def get_items_by_group(self, board_id, group_id):
        query = get_items_by_group_query(board_id=board_id, group_id=group_id)
        return self.client.execute(query)

    def create_group(self, board_id, group_name):
        query = create_group_query(board_id=board_id, group_name=group_name)
        return self.client.execute(query)

    def duplicate_group(self, board_id, group_id):
        query = duplicate_group_query(board_id=board_id, group_id=group_id)
        return self.client.execute(query)

    def archive_group(self, board_id, group_id):
        query = archive_group_query(board_id=board_id, group_id=group_id)
        return self.client.execute(query)

    def delete_group(self, board_id, group_id):
        query = delete_group_query(board_id=board_id, group_id=group_id)
        return self.client.execute(query)



from typing import Optional

from .base import BaseResource
from ..query_joins import (create_webhook_query, delete_webhook_query)


class WebhooksResource(BaseResource):
    def __init__(self, token):
        super().__init__(token)

    def create_webhook(self, board_id, url, event, column_id: Optional[str]):
        query = create_webhook_query(board_id, url, event, column_id)
        return self.client.execute(query)
    
    def delete_webhook(self, webhook_id):
        query = delete_webhook_query(webhook_id)
        return self.client.execute(query)
    